/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.framework.core;

/**
 * This class represents an authentication challenge element. It  corresponds
 * to the &lt;Chal&gt; element in SyncML representation DTD
 *
 *
 *
 *  @version $Id: Chal.java,v 1.2 2006/06/09 16:37:25 nichele Exp $
 */
public class Chal
implements java.io.Serializable {

    // ------------------------------------------------------------ Private data
    private Meta meta = null;

    // ------------------------------------------------------------ Constructors
    /** For serialization purposes */
    protected Chal() {}

    /**
     * Constructs a new Chal object.
     *
     * @param meta The meta object - NOT NULL
     *
     */
    public Chal (final Meta meta) {
        this.meta = meta;
        String type         = meta.getType();
        String format       = meta.getFormat();

        if (type == null) {
            throw new IllegalArgumentException(
                                     "The authentication type cannot be null");
        }
        if (format == null) {
            if (type.equalsIgnoreCase(Cred.AUTH_TYPE_BASIC)) {
                meta.setFormat(Constants.FORMAT_B64);
            } else if (type.equalsIgnoreCase(Cred.AUTH_TYPE_MD5)) {
                meta.setFormat(Constants.FORMAT_B64);
            } else {
                throw new IllegalArgumentException(
                                   "The authentication format cannot be null");
            }
        }
    }

    // ---------------------------------------------------------- Public methods
    /**
     * Gets the Meta property
     *
     * @return meta the Meta property
     */
    public Meta getMeta() {
        return this.meta;
    }

    /**
     * Sets the Meta property
     *
     * @param meta the Meta property
     *
     */
    public void setMeta(Meta meta) {
        this.meta = meta;
    }

    /**
     * Returns the nextNonce property or null
     *
     *  @return the nextNonce property or null
     */
    public NextNonce getNextNonce() {
        return meta.getNextNonce();
    }

    public void setNextNonce(NextNonce nextNonce) {
        if (meta == null) {
            meta = new Meta();
        }
        meta.setNextNonce(nextNonce);
    }
    /**
     * Returns the authentication type
     *
     * @return authentication type.
     */
    public String getType() {
        return meta.getType();
    }

    /**
     * Returns the authentication format
     *
     * @return format the authentication format
     */
    public String getFormat() {
        return meta.getFormat();
    }

    /**
     * Creates a basic authentication challange.
     * This will have type = Cred.AUTH_TYPE_BASIC and
     * format = Constants.FORMAT_B64
     *
     * @return the newly created AuthenticationChallange
     */
    public static Chal getBasicChal() {
        Meta m = new Meta();
        m.setType(Cred.AUTH_TYPE_BASIC);
        m.setFormat(Constants.FORMAT_B64);
        m.setNextNonce(null);
        return new Chal(m);
    }

    /**
     * Creates a MD5 authentication challange.
     * This will have type = Cred.AUTH_TYPE_MD5 and
     * format = Constants.FORMAT_B64
     *
     * @return the newly created AuthenticationChallange
     */
    public static Chal getMD5Chal() {
        Meta m = new Meta();
        m.setType(Cred.AUTH_TYPE_MD5);
        m.setFormat(Constants.FORMAT_B64);
        m.setNextNonce(null);
        return new Chal(m);
    }
}
