/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.framework.engine.source;

import com.funambol.framework.engine.SyncItem;
import com.funambol.framework.engine.SyncItemKey;


/**
 * A <code>SyncSource</code> that supports the filtering.
 *
 * @version $Id: FilterableSyncSource.java,v 1.1 2006/02/17 10:35:13 nichele Exp $
 */
public interface FilterableSyncSource extends SyncSource {

    // ---------------------------------------------------------- Public Methods

    /**
     * Called to get the state of the item with the given key.
     *
     * @return char the state of the item with the given
     *         key. If no item is found, <code>SyncItemState.NOT_EXISTING</code>
     *         has to be returned.
     *
     * @param syncItemKey the key of the SyncItem to return
     *
     * @throws SyncSourceException in case of errors (for instance if the
     *         underlying data store runs into problems)
     */
    public char getSyncItemStateFromId(SyncItemKey syncItemKey) throws SyncSourceException;

    /**
     * Called to check if the given item satisfies the filter clause specified
     * in the beginSync
     *
     * @param item the item to check
     * @return true is the item satisfies the filter, false otherwise
     * @throws SyncSourceException
     */
    public boolean isSyncItemInFilterClause(SyncItem item) throws SyncSourceException;


    /**
     * Called to check if the item with the given key satisfies the filter
     * clause specified in the beginSync
     * @param item the item to check
     * @return true is the item satisfies the fiter, false otherwise
     * @throws SyncSourceException if the item with the given key doesn't exist or
     *                             if an error occurs
     */
    public boolean isSyncItemInFilterClause(SyncItemKey key) throws SyncSourceException;

}
