/**
 * Copyright (C) 2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.framework.engine.source;

import com.funambol.framework.filter.FilterClause;
import com.funambol.framework.security.Sync4jPrincipal;

/**
 * Contains the context used in the synchronization process.
 * @version $Id: SyncContext.java,v 1.3 2006/04/05 19:43:23 nichele Exp $
 */
public class SyncContext implements java.io.Serializable {

    // --------------------------------------------------------------- Constants
    public static final int CONFLICT_RESOLUTION_SERVER_WINS = 0;
    public static final int CONFLICT_RESOLUTION_CLIENT_WINS = 1;
    public static final int CONFLICT_RESOLUTION_MERGE_DATA  = 2;

    // -------------------------------------------------------------- Properties

    private Sync4jPrincipal principal          = null;
    private int             syncMode           = -1;
    private FilterClause    filterClause       = null;
    private String          sourceQuery        = null;
    private int             conflictResolution = 0;

    /**
     * Creates a new SyncContext
     * @param principal the principal
     * @param syncMode the mode
     * @param filter the filter
     */
    public SyncContext(Sync4jPrincipal principal  ,
                       int             syncMode   ,
                       FilterClause    filter     ,
                       String          sourceQuery,
                       int             conflictResolution) {

        this.principal          = principal;
        this.syncMode           = syncMode;
        this.filterClause       = filter;
        this.sourceQuery        = sourceQuery;
        this.conflictResolution = conflictResolution;
    }

    /**
     * Returns the principal
     * @return the principal
     */
    public Sync4jPrincipal getPrincipal() {
        return principal;
    }

    /**
     * Returns the syncMode
     * @return the syncMode
     */
    public int getSyncMode() {
        return syncMode;
    }

    /**
     * Returns the filterClause
     * @return the filterClause
     */
    public FilterClause getFilterClause() {
        return filterClause;
    }

    /**
     * Returns the sourceQuery
     * @return the sourceQuery
     */
    public String getSourceQuery() {
        return sourceQuery;
    }

    /**
     * Returns the conflictResolution
     * @return the conflictResolution
     */
    public int getConflictResolution() {
        return conflictResolution;
    }
}
