/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.framework.notification.builder;

import com.funambol.framework.core.Alert;
import com.funambol.framework.notification.NotificationException;

/**
 * The implementations of this interface must build the notification message.
 *
 * @version $Id: DSNotificationMessageBuilder.java,v 1.1 2006/01/18 17:00:07 luigiafassina Exp $
 */
public interface DSNotificationMessageBuilder {
    
    // ---------------------------------------------------------- Public Methods
    
    /**
     * Builds the notification message with the given information
     *
     * @param sessionId       the identifier of the session associated with the 
     *                        Notification Package
     * @param deviceAddress   the address or phone number (depending on the 
     *                        implementation) of the device where the message 
     *                        should be sent
     * @param serverId        the id of the server
     * @param serverPw        server's password
     * @param serverNonce     server's nonce
     * @param alerts          array of Alert command for the datastore to sync, 
     *                        used to create the binary message to send
     * @param protocolVersion the version of SyncML protocol
     *
     * @return array of byte containing the notification message
     * @throws NotificationException
     */
    byte[] buildMessage(int     sessionId      ,
                        String  deviceAddress  ,
                        String  serverId       ,
                        String  serverPw       ,
                        byte[]  serverNonce    ,
                        Alert[] alerts         ,
                        int     uimode         ,
                        float   protocolVersion)
    throws NotificationException;    
}