/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 */


package com.funambol.framework.security;

import java.security.Principal;

import com.funambol.framework.core.Cred;

/**
 * This the interface that defines how credentials are authenticated to a device
 * and how resources are authorized for a given credential.
 *
 *.com
 * @version $Id: Officer.java,v 1.1 2006/01/18 17:00:08 luigiafassina Exp $
 */
public interface Officer {
    
    /**
     * Authenticates a credential.
     *
     * @param credential the credential to be authenticated
     *
     * @return true if the credential is autenticated, false otherwise
     */
    public boolean authenticate(Cred credential);
    
    /**
     * Un-authenticates a credential.
     * 
     * @param credential the credential to be unauthenticated
     */
    public void unAuthenticate(Cred credential);
    
    /**
     * Authorizes a resource.
     *
     * @param credential the requesting entity
     * @param resource the name (or the identifier) of the resource to be authorized
     *
     * @return true if the credential is authorized to access the resource, false
     *         otherwise
     */
    public boolean authorize(Principal credential, String resource);

    /**
     * Is the account authenticated by this officer expired?
     *
     * @return true if the account is expired, false otherwise
     */
    public boolean isAccountExpired();
    
    /**
     * Read property client_auth
     *
     * @return the authentication's type the server requires to client
     *         to authenticate it
     */
    public String getClientAuth();
    
    /**
     * Read property server_auth
     *
     * @return the authentication's type the client requires to server to 
     *         authenticate it
     */
    public String getServerAuth();
}

