/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307  USA
 */
package com.funambol.framework.tools;

import com.funambol.framework.logging.Sync4jLogger;

import javax.naming.InitialContext;
import javax.naming.NameNotFoundException;
import javax.naming.NamingException;
import javax.sql.DataSource;
import java.util.logging.Logger;

/**
 * This class provides implementation for the datasource name lookup
 *
 * @version $Id: DataSourceTools.java,v 1.1 2006/01/18 17:00:11 luigiafassina Exp $
 */
public class DataSourceTools {


    // ---------------------------------------------------------- Public methods

    /**
     * Performs datasource lookup using the given dataSourceName and context.
     * This class is usefull because given a datasource name like 'java:/jdbc/fnblds' it
     * tries with:
     * <ui>
     * <li>jdbc/fnblds</li>
     * <li>java:/jdbc/fnblds</li>
     * <li>java:/comp/env/jdbc/fnblds</li>
     * </ui>
     *
     * The algorithm used is: given the datasource name, checks if it starts with
     * 'java:/comp/env/' or with 'java:/'. If this happen, the "base" datasource name is
     * obtained removing 'java:/comp/env/' or 'java:/'.
     * Using the "base_datasource", tries to lookup:
     * <ui>
     * <li>base_datasource</li>
     * <li>java:/base_datasource</li>
     * <li>java:/comp/env/base_datasource</li>
     * </ui>
     *
     * @param dataSourceName datasource name that has to be looked up
     * @param context        initial context object used for lookup process
     */
    public static DataSource lookupDataSource(String dataSourceName,
                                              InitialContext context)
    throws NamingException {

        DataSource dataSource = null;

        if (dataSourceName == null) {
            dataSource = null;
        }

        //
        // We first try with the name as it is; if we get a NameNotFound
        // exception than we try prepending 'java:/comp/env/'
        //
        if (dataSourceName.startsWith("java:/comp/env/")) {
            dataSourceName = (dataSourceName.length() > 15)
                    ? dataSourceName.substring(15)
                    : dataSourceName
                    ;
        }
        if (dataSourceName.startsWith("java:/")) {
            dataSourceName = (dataSourceName.length() > 6)
                    ? dataSourceName.substring(6)
                    : dataSourceName
                    ;
        }


        try {
            dataSource = (DataSource) context.lookup(dataSourceName);
        } catch (NameNotFoundException e) {
            try {
                dataSource = (DataSource) context.lookup("java:/" +
                        dataSourceName);
            } catch (NameNotFoundException nNFE) {

                dataSource = (DataSource) context.lookup("java:/comp/env/" +
                        dataSourceName);
            }
        }

        return dataSource;
    }

}
