/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.funambol.server.store;

import java.util.ArrayList;
import java.sql.*;
import javax.sql.DataSource;

import com.funambol.framework.tools.DBTools;
import com.funambol.framework.server.store.*;

/**
 * This class is base class for SQL storage. Each extension of this
 * class MUST respect the usage of the first 2 queries are for the
 * counter.
 *
 * @version $Id: UtilsPersistentStore.java,v 1.1 2006/01/18 17:00:12 luigiafassina Exp $
 *
 */
public class UtilsPersistentStore {

    // --------------------------------------------------------------- Constants

    public static final int SQL_GET_COUNTER                  = 0;
    public static final int SQL_UPDATE_COUNTER               = 1;

    // -------------------------------------------------------------- Properties

    protected String idSpace = null;

    public void setIdSpace(String idSpace) {
        this.idSpace = idSpace;
    }

    public String getIdSpace() {
        return this.idSpace;
    }

    protected String[] sql = null;

    public void setSql(String[] sql) {
        this.sql = sql;
    }

    public String[] getSql() {
        return this.sql;
    }

    // ------------------------------------------------------------ Private data
    // ------------------------------------------------------------ Constructors
    // ---------------------------------------------------------- Public methods

    /**
     * Read the counter of the specific id in order to use it like primary key
     * in case of inserting a new record into data store.
     *
     * @throws PersistentException in case of error reading the data store
     */
    protected int readCounter(Connection conn) throws PersistentStoreException {
        PreparedStatement stmt = null;
        ResultSet rs = null;
        int counter = 0;

        try {
            stmt = conn.prepareStatement(sql[SQL_GET_COUNTER]);
            rs = stmt.executeQuery();

            if (rs.next() == false) {
                throw new NotFoundException("Counter not found for "
                + idSpace);
            }
            counter = rs.getInt(1) + 1;

            rs.close(); rs = null;
            stmt.close(); stmt = null;

            stmt = conn.prepareStatement(sql[SQL_UPDATE_COUNTER]);
            stmt.setInt(1, counter);
            stmt.executeUpdate();

        } catch (SQLException e) {
            e.printStackTrace();
            throw new PersistentStoreException("Error reading the counter " + counter, e);
        } finally {
            DBTools.close(null, stmt, rs);
        }
        return counter;
    }
}

